@@ -26,7 +26,7 @@ is tricked into clicking on the link in their browser.
     vulnerability.
 
 2.  If the corresponding view function is decorated with
-    `ext.perms.protected` the CSFR token will be automatically
+    `@protected()` the CSFR token will be automatically
     checked. If no CSFR token is found or its value is incorrect, the
     decorator will raise a :class:`shake.NotAllowed` error.
 
@@ -86,6 +86,7 @@ send back the CSRF token in the custom X-CSRFTOKEN header:
 
 """
 from shake import redirect, NotAllowed, url_for, get_csrf
+from werkzeug.exceptions import HTTPException
 
 
 __all__ = (
@@ -106,17 +107,12 @@ def _login_required(request, sign_in_url=None, redirect_to=None):
     return redirect(sign_in_url)
 
 
-def protected(perm=None, group=None, test=None, sign_in_url=None, csrf=None):
+def protected(test=None, sign_in_url=None, csrf=None):
     """Factory of decorators for limit the access to views.
     
-    perm
-    :   permission the user must have.
-    
     test
-    :   A function that takes the request and has to returns True or False.
-    
-    group
-    :   Group the user must be in.
+    :   A function that takes the request and returns `True` or `False`. If it
+        return `False`, a `NotAllowed` exception is raised.
     
     sign_in_url
     :   If any required condition fail, redirect to this place.
@@ -138,18 +134,6 @@ def protected(perm=None, group=None, test=None, sign_in_url=None, csrf=None):
             if not user:
                 return _login_required(request, sign_in_url)
             
-            no_perm = (perm is not None) \
-                and not (hasattr(user, 'permissions') \
-                and perm in user.permissions)
-            if no_perm:
-                raise NotAllowed()
-            
-            no_group = (group is not None) \
-                and not (hasattr(user, 'groups') \
-                and group in user.groups)
-            if no_group:
-                raise NotAllowed()
-            
             test_fail = (test is not None) and (not test(request, **kwargs))
             if test_fail:
                 raise NotAllowed()
@@ -180,9 +164,15 @@ def is_valid(request, csrf=True):
 
 def invalid_csrf(request):
     valid_csrf_token = get_csrf(request)
-    csrf_token = request.values.get(
-        valid_csrf_token.name,
-        request.headers.get('X-CSRFToken'))
+    try:
+        csrf_token = request.values.get(
+            valid_csrf_token.name,
+            request.headers.get('X-CSRFToken'))
+    except HTTPException, e:
+        try:
+            csrf_token = request.headers.get('X-CSRFToken')
+        except HTTPException, e:
+            return True
     return csrf_token != valid_csrf_token.value
 
 invalid_csrf_secret = invalid_csrf
